MODULE trcbio_medusa
   !!======================================================================
   !!                         ***  MODULE trcbio  ***
   !! TOP :   MEDUSA
   !!======================================================================
   !! History :
   !!  -   !  1999-07  (M. Levy)              original code
   !!  -   !  2000-12  (E. Kestenare)         assign parameters to name individual tracers
   !!  -   !  2001-03  (M. Levy)              LNO3 + dia2d 
   !! 2.0  !  2007-12  (C. Deltel, G. Madec)  F90
   !!  -   !  2008-08  (K. Popova)            adaptation for MEDUSA
   !!  -   !  2008-11  (A. Yool)              continuing adaptation for MEDUSA
   !!  -   !  2010-03  (A. Yool)              updated for branch inclusion
   !!  -   !  2011-08  (A. Yool)              updated for ROAM (see below)
   !!  -   !  2013-03  (A. Yool)              updated for iMARNET
   !!  -   !  2013-05  (A. Yool)              updated for v3.5
   !!  -   !  2014-08  (A. Yool, J. Palm)     Add DMS module for UKESM1 model
   !!  -   !  2015-06  (A. Yool)              Update to include MOCSY
   !!  -   !  2015-07  (A. Yool)              Update for rolling averages
   !!  -   !  2015-10  (J. Palm)              Update for diag outputs through iom_use  
   !!  -   !  2016-11  (A. Yool)              Updated diags for CMIP6
   !!----------------------------------------------------------------------
   !!
#if defined key_roam
   !!----------------------------------------------------------------------
   !! Updates for the ROAM project include:
   !!   - addition of DIC, alkalinity, detrital carbon and oxygen tracers
   !!   - addition of air-sea fluxes of CO2 and oxygen
   !!   - periodic (monthly) calculation of full 3D carbonate chemistry 
   !!   - detrital C:N ratio now free to evolve dynamically
   !!   - benthic storage pools
   !! 
   !! Opportunity also taken to add functionality:
   !!   - switch for Liebig Law (= most-limiting) nutrient uptake
   !!   - switch for accelerated seafloor detritus remineralisation
   !!   - switch for fast -> slow detritus transfer at seafloor
   !!   - switch for ballast vs. Martin vs. Henson fast detritus remin.
   !!   - per GMD referee remarks, xfdfrac3 introduced for grazed PDS
   !!----------------------------------------------------------------------
#endif
   !!
#if defined key_mocsy
   !!----------------------------------------------------------------------
   !! Updates with the addition of MOCSY include:
   !!   - option to use PML or MOCSY carbonate chemistry (the latter is 
   !!     preferred)
   !!   - central calculation of gas transfer velocity, f_kw660; previously
   !!     this was done separately for CO2 and O2 with predictable results
   !!   - distribution of f_kw660 to both PML and MOCSY CO2 air-sea flux 
   !!     calculations and to those for O2 air-sea flux
   !!   - extra diagnostics included for MOCSY
   !!----------------------------------------------------------------------
#endif
   !!
#if defined key_medusa
   !!----------------------------------------------------------------------
   !!                                        MEDUSA bio-model
   !!----------------------------------------------------------------------
   !!   trc_bio_medusa        :  
   !!----------------------------------------------------------------------
      USE oce_trc
      USE trc
      USE sms_medusa
      USE lbclnk
      USE prtctl_trc      ! Print control for debugging
      USE trcsed_medusa
      USE sbc_oce         ! surface forcing
      USE sbcrnf          ! surface boundary condition: runoff variables
      USE in_out_manager  ! I/O manager
# if defined key_iomput
      USE iom
      USE trcnam_medusa         ! JPALM 13-11-2015 -- if iom_use for diag
      !!USE trc_nam_iom_medusa  ! JPALM 13-11-2015 -- if iom_use for diag
# endif
# if defined key_roam
      USE gastransfer
#  if defined key_mocsy
      USE mocsy_wrapper
#  else
      USE trcco2_medusa
#  endif
      USE trcoxy_medusa
      !! Jpalm (08/08/2014)
      USE trcdms_medusa
# endif
      !! AXY (18/01/12): brought in for benthic timestepping
      USE trcnam_trp      ! AXY (24/05/2013)
      USE trdmxl_trc
      USE trdtrc_oce  ! AXY (24/05/2013)

      !! AXY (30/01/14): necessary to find NaNs on HECTOR
      USE, INTRINSIC :: ieee_arithmetic 

      !! JPALM (27-06-2016): add lk_oasis for CO2 and DMS coupling with atm
      USE sbc_oce,                    ONLY: lk_oasis
      USE oce,                        ONLY: CO2Flux_out_cpl, DMS_out_cpl,   &
                                            PCO2a_in_cpl
      USE bio_medusa_mod
      USE bio_medusa_init_mod,        ONLY: bio_medusa_init
      USE carb_chem_mod,              ONLY: carb_chem
      USE air_sea_mod,                ONLY: air_sea
      USE plankton_mod,               ONLY: plankton
      USE iron_chem_scav_mod,         ONLY: iron_chem_scav
      USE detritus_mod,               ONLY: detritus
      USE bio_medusa_update_mod,      ONLY: bio_medusa_update
      USE bio_medusa_diag_slice_mod,  ONLY: bio_medusa_diag_slice
      USE bio_medusa_fin_mod,         ONLY: bio_medusa_fin

      IMPLICIT NONE
      PRIVATE
      
      PUBLIC   trc_bio_medusa    ! called in trcsms_medusa.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Id$
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_bio_medusa( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE trc_bio  ***
      !!
      !! ** Purpose :   compute the now trend due to biogeochemical processes
      !!              and add it to the general trend of passive tracers equations
      !!
      !! ** Method  :   each now biological flux is calculated in function of now
      !!              concentrations of tracers.
      !!              depending on the tracer, these fluxes are sources or sinks.
      !!              the total of the sources and sinks for each tracer
      !!              is added to the general trend.
      !!        
      !!                      tra = tra + zf...tra - zftra...
      !!                                     |         |
      !!                                     |         |
      !!                                  source      sink
      !!        
      !!              IF 'key_trc_diabio' defined , the biogeochemical trends
      !!              for passive tracers are saved for futher diagnostics.
      !!---------------------------------------------------------------------
      !!
      !!
      !!----------------------------------------------------------------------            
      !! Variable conventions
      !!----------------------------------------------------------------------
      !!
      !! names: z*** - state variable 
      !!        f*** - function (or temporary variable used in part of a function)
      !!        x*** - parameter
      !!        b*** - right-hand part (sources and sinks)
      !!        i*** - integer variable (usually used in yes/no flags)
      !!
      !! time (integer timestep)
      INTEGER, INTENT( in ) ::    kt
      !!
      !! spatial array indices
      INTEGER  ::    ji,jj,jk,jn
      !!
      !! AXY (27/07/10): add in indices for depth horizons (for sinking flux
      !!                 and seafloor iron inputs)
      !! INTEGER  ::    i0100, i0200, i0500, i1000, i1100
      !!
      !! model state variables
!      REAL(wp), DIMENSION(jpi,jpj) ::    zchn,zchd,zphn,zphd,zpds,zzmi
!      REAL(wp), DIMENSION(jpi,jpj) ::    zzme,zdet,zdtc,zdin,zsil,zfer
! zage doesn't seem to be used - marc 19/4/17
!      REAL(wp) ::    zage
!# if defined key_roam
!      REAL(wp), DIMENSION(jpi,jpj) ::    zdic, zalk, zoxy
!      REAL(wp), DIMENSION(jpi,jpj) ::    ztmp, zsal
!# endif
!# if defined key_mocsy
!      REAL(wp), DIMENSION(jpi,jpj) ::    zpho
!# endif
      !!
      !! integrated source and sink terms
!      REAL(wp) ::    b0
      !! AXY (23/08/13): changed from individual variables for each flux to
      !!                 an array that holds all fluxes
      REAL(wp), DIMENSION(jpi,jpj,jp_medusa) ::    btra
      !!
      !! primary production and chl related quantities      
!      REAL(wp), DIMENSION(jpi,jpj) ::    fthetan,faln,fchn1,fchn,fjln,fprn,frn
!      REAL(wp), DIMENSION(jpi,jpj) ::    fthetad,fald,fchd1,fchd,fjld,fprd,frd
      !! AXY (23/11/16): add in light-only limitation term (normalised 0-1 range)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fjlim_pn, fjlim_pd
      !! AXY (03/02/11): add in Liebig terms
!      REAL(wp), DIMENSION(jpi,jpj) ::    fpnlim, fpdlim
      !! AXY (16/07/09): add in Eppley curve functionality
!      REAL(wp), DIMENSION(jpi,jpj) ::    fun_T,xvpnT,xvpdT
      INTEGER  ::    ieppley
      !! AXY (16/05/11): per Katya's prompting, add in new T-dependence
      !!                 for phytoplankton growth only (i.e. no change
      !!                 for remineralisation)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fun_Q10
      !! AXY (01/03/10): add in mixed layer PP diagnostics
!      REAL(wp), DIMENSION(jpi,jpj) ::    fprn_ml,fprd_ml
      !!
      !! nutrient limiting factors
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnln,ffln2            !! N and Fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnld,ffld,fsld,fsld2 !! N, Fe and Si
      !!
      !! silicon cycle
!      REAL(wp), DIMENSION(jpi,jpj) ::    fsin,fnsi,fprds,fsdiss
      REAL(wp)                     ::    fsin1,fnsi1,fnsi2
      !!
      !! iron cycle; includes parameters for Parekh et al. (2005) iron scheme
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffetop,ffebot,ffescav
      REAL(wp) ::    xLgF, xFeT, xFeF, xFeL         !! state variables for iron-ligand system
!      REAL(wp), DIMENSION(jpi,jpj) ::  xFree        !! state variables for iron-ligand system
      REAL(wp) ::    xb_coef_tmp, xb2M4ac           !! iron-ligand parameters
      REAL(wp) ::    xmaxFeF,fdeltaFe               !! max Fe' parameters
      !!
      !! local parameters for Moore et al. (2004) alternative scavenging scheme
      REAL(wp) ::    fbase_scav,fscal_sink,fscal_part,fscal_scav
      !!
      !! local parameters for Moore et al. (2008) alternative scavenging scheme
      REAL(wp) ::    fscal_csink,fscal_sisink,fscal_casink
      !!
      !! local parameters for Galbraith et al. (2010) alternative scavenging scheme
      REAL(wp) ::    xCscav1, xCscav2, xk_org, xORGscav  !! organic portion of scavenging
      REAL(wp) ::    xk_inorg, xINORGscav                !! inorganic portion of scavenging
      !!
      !! microzooplankton grazing
!      REAL(wp), DIMENSION(jpi,jpj) ::    fmi1,fmi,fgmipn,fgmid,fgmidc
!      REAL(wp), DIMENSION(jpi,jpj) ::    finmi,ficmi,fstarmi,fmith,fmigrow,fmiexcr,fmiresp
      !!
      !! mesozooplankton grazing
!      REAL(wp), DIMENSION(jpi,jpj) ::    fme1,fme,fgmepn,fgmepd,fgmepds,fgmezmi,fgmed,fgmedc
!      REAL(wp), DIMENSION(jpi,jpj) ::    finme,ficme,fstarme,fmeth,fmegrow,fmeexcr,fmeresp
      !!
      !! mortality/Remineralisation (defunct parameter "fz" removed)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fdpn,fdpd,fdpds,fdzmi,fdzme,fdd
# if defined key_roam
!      REAL(wp), DIMENSION(jpi,jpj) ::    fddc
# endif
!      REAL(wp), DIMENSION(jpi,jpj) ::    fdpn2,fdpd2,fdpds2,fdzmi2,fdzme2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fslown, fslowc
!      REAL(wp), DIMENSION(jpi,jpj) ::    fslownflux, fslowcflux
      REAL(wp), DIMENSION(jpi,jpj) ::    fregen,fregensi
!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfast,fregenfastsi
# if defined key_roam
!! Doesn't look like this is used - marc 10/4/17
!!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenc
!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfastc
# endif
      !!
      !! particle flux
!      REAL(WP), DIMENSION(jpi,jpj) ::    fdep1,fcaco3
!      REAL(WP), DIMENSION(jpi,jpj) ::    ftempn,ftempsi,ftempfe,ftempc,ftempca
!      REAL(wp), DIMENSION(jpi,jpj) ::    freminn,freminsi,freminfe,freminc,freminca
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffastn,ffastsi,ffastfe,ffastc,ffastca
!      REAL(wp), DIMENSION(jpi,jpj) ::    fprotf
!      REAL(wp), DIMENSION(jpi,jpj) ::    fsedn,fsedsi,fsedfe,fsedc,fsedca
!      REAL(wp), DIMENSION(jpi,jpj) ::    fccd
!      REAL(wp), DIMENSION(jpi,jpj) ::    fccd_dep
      !!
      !! AXY (06/07/11): alternative fast detritus schemes
      REAL(wp) ::    fb_val, fl_sst
      !!
      !! AXY (08/07/11): fate of fast detritus reaching the seafloor
! I don't think ffast2slowfe is used - marc 10/4/17
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffast2slown,ffast2slowfe,ffast2slowc
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffast2slown,ffast2slowc
      !!
      !! conservation law
      REAL(wp) ::    fnit0,fsil0,ffer0 
# if defined key_roam
      REAL(wp) ::    fcar0,falk0,foxy0 
# endif      
      !! 
      !! temporary variables
      REAL(wp) ::    fq0,fq1,fq2,fq3,fq4,fq5,fq6,fq7,fq8,fq9
      !!
      !! water column nutrient and flux integrals
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_n,ftot_si,ftot_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_n,fflx_si,fflx_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_n,fifd_si,fifd_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_n,fofd_si,fofd_fe
# if defined key_roam
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_c,ftot_a,ftot_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_c,fflx_a,fflx_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_c,fifd_a,fifd_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_c,fofd_a,fofd_o2
# endif
      !!
      !! zooplankton grazing integrals
!      REAL(wp), DIMENSION(jpi,jpj) ::    fzmi_i,fzmi_o,fzme_i,fzme_o
      !!
      !! limitation term temporary variables
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_pn,ftot_pd
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_zmi,ftot_zme,ftot_det,ftot_dtc
      !! use ballast scheme (1) or simple exponential scheme (0; a conservation test)
      INTEGER  ::    iball
      !! use biological fluxes (1) or not (0)
!      INTEGER  ::    ibio_switch
      !!
      !! diagnose fluxes (should only be used in 1D runs)
!      INTEGER  ::    idf, idfval
      !!
      !! nitrogen and silicon production and consumption
      REAL(wp) ::    fn_prod, fn_cons, fs_prod, fs_cons
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnit_prod, fnit_cons, fsil_prod, fsil_cons
# if defined key_roam
      !!
      !! flags to help with calculating the position of the CCD
! Moved into carb_chem.F90 - marc 20/4/17
!      INTEGER, DIMENSION(jpi,jpj) ::     i2_omcal,i2_omarg
      !!
      !! AXY (24/11/16): add xCO2 variable for atmosphere (what we actually have)
!      REAL(wp)                     ::    f_xco2a
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_ph, f_pco2w, f_h2co3, f_hco3, f_co3, f_co2flux
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_TDIC, f_TALK, f_dcf, f_henry
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_pp0
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_kw660, f_o2flux, f_o2sat
      REAL(wp)                     ::    f_o2sat3
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_omcal, f_omarg
      !!
      !! AXY (23/06/15): additional diagnostics for MOCSY and oxygen
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_fco2w, f_BetaD, f_rhosw, f_opres, f_insitut, f_pco2atm, f_fco2atm
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_schmidtco2, f_kwco2, f_K0, f_co2starair, f_dpco2, f_kwo2
      !! jpalm 14-07-2016: convert CO2flux diag from mmol/m2/d to kg/m2/s
      REAL, PARAMETER :: weight_CO2_mol = 44.0095  !! g / mol
      REAL, PARAMETER :: secs_in_day    = 86400.0  !! s / d
      REAL, PARAMETER :: CO2flux_conv   = (1.e-6 * weight_CO2_mol) / secs_in_day

      !!
!      INTEGER, DIMENSION(jpi,jpj)  ::    iters
      REAL(wp) ::    f_year
      INTEGER  ::    i_year
      INTEGER  ::    iyr1, iyr2
      !!
      !! carbon, alkalinity production and consumption
      REAL(wp) ::    fc_prod, fc_cons, fa_prod, fa_cons
!      REAL(wp), DIMENSION(jpi,jpj) ::    fcomm_resp
!      REAL(wp), DIMENSION(jpi,jpj) ::    fcar_prod, fcar_cons
      !!
      !! oxygen production and consumption (and non-consumption)
      REAL(wp), DIMENSION(jpi,jpj) ::    fo2_prod, fo2_cons, fo2_ncons, fo2_ccons
!      REAL(wp), DIMENSION(jpi,jpj) ::    foxy_prod, foxy_cons, foxy_anox
      !! Jpalm (11-08-2014)
      !! add DMS in MEDUSA for UKESM1 model
!      REAL(wp), DIMENSION(jpi,jpj) ::    dms_surf
      !! AXY (13/03/15): add in other DMS calculations
!      REAL(wp), DIMENSION(jpi,jpj) ::    dms_andr, dms_simo, dms_aran, dms_hall

# endif
      !! 
      !! benthic fluxes
!      INTEGER  ::    ibenthic
!      REAL(wp), DIMENSION(jpi,jpj) :: f_sbenin_n, f_sbenin_fe,              f_sbenin_c
!      REAL(wp), DIMENSION(jpi,jpj) :: f_fbenin_n, f_fbenin_fe, f_fbenin_si, f_fbenin_c, f_fbenin_ca
!      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_n, f_benout_fe, f_benout_si, f_benout_c, f_benout_ca
      REAL(wp) ::    zfact
      !! 
      !! benthic fluxes of CaCO3 that shouldn't happen because of lysocline
!      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_lyso_ca
      !!
      !! riverine fluxes
!      REAL(wp), DIMENSION(jpi,jpj) :: f_runoff, f_riv_n, f_riv_si, f_riv_c, f_riv_alk
      !! AXY (19/07/12): variables for local riverine fluxes to handle inputs below surface
!      REAL(wp), DIMENSION(jpi,jpj) :: f_riv_loc_n, f_riv_loc_si, f_riv_loc_c, f_riv_loc_alk
      !!---------------------------------------------------------------------

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: variables defined'
      CALL flush(numout)
# endif 

      !! AXY (20/11/14): alter this to report on first MEDUSA call
      !! IF( kt == nit000 ) THEN
      IF( kt == nittrc000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' trc_bio: MEDUSA bio-model'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~'
	 IF(lwp) WRITE(numout,*) ' kt =',kt
      ENDIF

      !! AXY (13/01/12): is benthic model properly interactive? 0 = no, 1 = yes
      ibenthic = 1

      !! not sure what this is for; it's not used anywhere; commenting out
      !! fbodn(:,:) = 0.e0   

      !!
      IF( ln_diatrc ) THEN
         !! blank 2D diagnostic array
         trc2d(:,:,:) = 0.e0
         !!
         !! blank 3D diagnostic array
         trc3d(:,:,:,:) = 0.e0
      ENDIF

      !!----------------------------------------------------------------------
      !! b0 is present for debugging purposes; using b0 = 0 sets the tendency
      !! terms of all biological equations to 0.
      !!----------------------------------------------------------------------
      !!
      !! AXY (03/09/14): probably not the smartest move ever, but it'll fit
      !!                 the bill for now; another item on the things-to-sort-
      !!		 out-in-the-future list ...
# if defined key_kill_medusa
      b0 = 0.
# else
      b0 = 1.
# endif
      !!----------------------------------------------------------------------
      !! fast detritus ballast scheme (0 = no; 1 = yes)
      !! alternative to ballast scheme is same scheme but with no ballast
      !! protection (not dissimilar to Martin et al., 1987)
      !!----------------------------------------------------------------------
      !!
      iball = 1

      !!----------------------------------------------------------------------
      !! full flux diagnostics (0 = no; 1 = yes); appear in ocean.output
      !! these should *only* be used in 1D since they give comprehensive
      !! output for ecological functions in the model; primarily used in
      !! debugging
      !!----------------------------------------------------------------------
      !!
      idf    = 0
      !!
      !! timer mechanism
      if (kt/120*120.eq.kt) then
         idfval = 1
      else
         idfval = 0
      endif

      !!----------------------------------------------------------------------
      !! Initialise arrays to zero and set up arrays for diagnostics
      !!----------------------------------------------------------------------
! tmp - marc
      write(numout,*) 'bbb13. before call to bio_medusa_init,kt=',kt
      flush(numout)
!
      CALL bio_medusa_init( kt )
! tmp - marc
      write(numout,*) 'bbb14. after call to bio_medusa_init,kt=',kt
      flush(numout)
!
       !!
# if defined key_axy_nancheck
       DO jn = 1,jptra
          !! fq0 = MINVAL(trn(:,:,:,jn))
          !! fq1 = MAXVAL(trn(:,:,:,jn))
          fq2 = SUM(trn(:,:,:,jn))
          !! if (lwp) write (numout,'(a,2i6,3(1x,1pe15.5))') 'NAN-CHECK', &
          !! &        kt, jn, fq0, fq1, fq2
          !! AXY (30/01/14): much to our surprise, the next line doesn't work on HECTOR
          !!                 and has been replaced here with a specialist routine
          !! if (fq2 /= fq2 ) then
          if ( ieee_is_nan( fq2 ) ) then
             !! there's a NaN here
             if (lwp) write(numout,*) 'NAN detected in field', jn, 'at time', kt, 'at position:'
             DO jk = 1,jpk
                DO jj = 1,jpj
                   DO ji = 1,jpi
                      !! AXY (30/01/14): "isnan" problem on HECTOR
                      !! if (trn(ji,jj,jk,jn) /= trn(ji,jj,jk,jn)) then
                      if ( ieee_is_nan( trn(ji,jj,jk,jn) ) ) then
                         if (lwp) write (numout,'(a,1pe12.2,4i6)') 'NAN-CHECK', &
                         &        tmask(ji,jj,jk), ji, jj, jk, jn
                      endif
                   enddo
                enddo
             enddo
             CALL ctl_stop( 'trcbio_medusa, NAN in incoming tracer field' )
          endif
       ENDDO
       CALL flush(numout)
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: variables initialised and checked'
      CALL flush(numout)
# endif 

# if defined key_roam
      !!----------------------------------------------------------------------
      !! calculate atmospheric pCO2
      !!----------------------------------------------------------------------
      !!
      !! what's atmospheric pCO2 doing? (data start in 1859)
      iyr1  = nyear - 1859 + 1
      iyr2  = iyr1 + 1
      if (iyr1 .le. 1) then
         !! before 1860
         f_xco2a(:,:) = hist_pco2(1)
      elseif (iyr2 .ge. 242) then
         !! after 2099
         f_xco2a(:,:) = hist_pco2(242)
      else
         !! just right
         fq0 = hist_pco2(iyr1)
         fq1 = hist_pco2(iyr2)
         fq2 = real(nsec_day) / (60.0 * 60.0 * 24.0)
         !! AXY (14/06/12): tweaked to make more sense (and be correct)
#  if defined key_bs_axy_yrlen
         fq3 = (real(nday_year) - 1.0 + fq2) / 360.0  !! bugfix: for 360d year with HadGEM2-ES forcing
#  else
         fq3 = (real(nday_year) - 1.0 + fq2) / 365.0  !! original use of 365 days (not accounting for leap year or 360d year)
#  endif
         fq4 = (fq0 * (1.0 - fq3)) + (fq1 * fq3)
         f_xco2a(:,:) = fq4
      endif
#  if defined key_axy_pi_co2
      f_xco2a(:,:) = 284.725          !! OCMIP pre-industrial pCO2
#  endif
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nyear     =', nyear
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nsec_day  =', real(nsec_day)
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nday_year =', real(nday_year)
      !! AXY (29/01/14): remove surplus diagnostics
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq0       =', fq0
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq1       =', fq1
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq2       =', fq2
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq3       =', fq3
      IF(lwp) WRITE(numout,*) ' MEDUSA atm pCO2  =', f_xco2a(1,1)
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: ready for carbonate chemistry'
      IF (lwp) write (numout,*) 'trc_bio_medusa: kt = ', kt
      IF (lwp) write (numout,*) 'trc_bio_medusa: nittrc000 = ', nittrc000
      CALL flush(numout)
# endif 

# if defined key_roam
      !! AXY (20/11/14): alter to call on first MEDUSA timestep and then every
      !!                 month (this is hardwired as 960 timesteps but should
      !!                 be calculated and done properly
      !! IF( kt == nit000 .or. mod(kt,1920) == 0 ) THEN
      !! IF( kt == nittrc000 .or. mod(kt,960) == 0 ) THEN 
      !!=============================
      !! Jpalm -- 07-10-2016 -- need to change carb-chem frequency call :
      !!          we don't want to call on the first time-step of all run submission, 
      !!          but only on the very first time-step, and then every month
      !!          So we call on nittrc000 if not restarted run, 
      !!          else if one month after last call.
      !!          assume one month is 30d --> 3600*24*30 : 2592000s
      !!          try to call carb-chem at 1st month's tm-stp : x * 30d + 1*rdt(i.e: mod = rdt)   
      !!          ++ need to pass carb-chem output var through restarts
      If ( ( kt == nittrc000 .AND. .NOT.ln_rsttr ) .OR. mod(kt*rdt,2592000.) == rdt ) THEN
         !!----------------------------------------------------------------------
         !! Calculate the carbonate chemistry for the whole ocean on the first
         !! simulation timestep and every month subsequently; the resulting 3D
         !! field of omega calcite is used to determine the depth of the CCD
         !!----------------------------------------------------------------------
         CALL carb_chem( kt )

      ENDIF
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: ready for full domain calculations'
      CALL flush(numout)
# endif 

      !!----------------------------------------------------------------------
      !! MEDUSA has unified equation through the water column
      !! (Diff. from LOBSTER which has two sets: bio- and non-bio layers) 
      !! Statement below in LOBSTER is different: DO jk = 1, jpkbm1          
      !!----------------------------------------------------------------------
      !!
      !! NOTE: the ordering of the loops below differs from that of some other
      !! models; looping over the vertical dimension is the outermost loop and
      !! this complicates some calculations (e.g. storage of vertical fluxes
      !! that can otherwise be done via a singular variable require 2D fields
      !! here); however, these issues are relatively easily resolved, but the
      !! loops CANNOT be reordered without potentially causing code efficiency
      !! problems (e.g. array indexing means that reordering the loops would
      !! require skipping between widely-spaced memory location; potentially
      !! outside those immediately cached)
      !!
      !! OPEN vertical loop
      DO jk = 1,jpk
         !! OPEN horizontal loops
         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then               
               !!===========================================================
               !! SETUP LOCAL GRID CELL
               !!===========================================================
               !!
               !!-----------------------------------------------------------
               !! Some notes on grid vertical structure
               !! - fsdepw(ji,jj,jk) is the depth of the upper surface of 
               !!   level jk
               !! - fsde3w(ji,jj,jk) is *approximately* the midpoint of 
               !!   level jk
               !! - fse3t(ji,jj,jk)  is the thickness of level jk
               !!-----------------------------------------------------------
               !!
               !! AXY (01/03/10): set up level depth (bottom of level)
               fdep1(ji,jj) = fsdepw(ji,jj,jk) + fse3t(ji,jj,jk)
               !! AXY (28/11/16): local seafloor depth
               !!                 previously mbathy(ji,jj) - 1, now 
               !!                 mbathy(ji,jj)
               mbathy(ji,jj) = mbathy(ji,jj)
               !!
               !! set up model tracers
               !! negative values of state variables are not allowed to
               !! contribute to the calculated fluxes
               !! non-diatom chlorophyll
               zchn(ji,jj) = max(0.,trn(ji,jj,jk,jpchn))
               !! diatom chlorophyll
               zchd(ji,jj) = max(0.,trn(ji,jj,jk,jpchd))
               !! non-diatoms
               zphn(ji,jj) = max(0.,trn(ji,jj,jk,jpphn))
               !! diatoms
               zphd(ji,jj) = max(0.,trn(ji,jj,jk,jpphd))
               !! diatom silicon
               zpds(ji,jj) = max(0.,trn(ji,jj,jk,jppds))
               !! AXY (28/01/10): probably need to take account of 
               !! chl/biomass connection
               if (zchn(ji,jj).eq.0.) zphn(ji,jj) = 0.
               if (zchd(ji,jj).eq.0.) zphd(ji,jj) = 0.
               if (zphn(ji,jj).eq.0.) zchn(ji,jj) = 0.
               if (zphd(ji,jj).eq.0.) zchd(ji,jj) = 0.
	       !! AXY (23/01/14): duh - why did I forget diatom silicon?
	       if (zpds(ji,jj).eq.0.) zphd(ji,jj) = 0.
	       if (zphd(ji,jj).eq.0.) zpds(ji,jj) = 0.
               !! microzooplankton
               zzmi(ji,jj) = max(0.,trn(ji,jj,jk,jpzmi))
               !! mesozooplankton
               zzme(ji,jj) = max(0.,trn(ji,jj,jk,jpzme))
               !! detrital nitrogen
               zdet(ji,jj) = max(0.,trn(ji,jj,jk,jpdet))
               !! dissolved inorganic nitrogen
               zdin(ji,jj) = max(0.,trn(ji,jj,jk,jpdin))
               !! dissolved silicic acid
               zsil(ji,jj) = max(0.,trn(ji,jj,jk,jpsil))
               !! dissolved "iron"
               zfer(ji,jj) = max(0.,trn(ji,jj,jk,jpfer))
# if defined key_roam
               !! detrital carbon
               zdtc(ji,jj) = max(0.,trn(ji,jj,jk,jpdtc))
               !! dissolved inorganic carbon
               zdic(ji,jj) = max(0.,trn(ji,jj,jk,jpdic))
               !! alkalinity
               zalk(ji,jj) = max(0.,trn(ji,jj,jk,jpalk))
               !! oxygen
               zoxy(ji,jj) = max(0.,trn(ji,jj,jk,jpoxy))
#  if defined key_axy_carbchem && defined key_mocsy
               !! phosphate via DIN and Redfield
               zpho(ji,jj) = max(0.,trn(ji,jj,jk,jpdin)) / 16.0
#  endif
               !!
               !! also need physical parameters for gas exchange calculations
               ztmp(ji,jj) = tsn(ji,jj,jk,jp_tem)
               zsal(ji,jj) = tsn(ji,jj,jk,jp_sal)
               !!
	       !! AXY (28/02/14): check input fields
               if (ztmp(ji,jj) .lt. -3.0 .or. ztmp(ji,jj) .gt. 40.0 ) then
                  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: T WARNING 2D, ',  &
                     tsb(ji,jj,jk,jp_tem), tsn(ji,jj,jk,jp_tem), ' at (',     &
                     ji, ',', jj, ',', jk, ') at time', kt
		  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: T SWITCHING 2D, ',&
                     tsn(ji,jj,jk,jp_tem), ' -> ', tsb(ji,jj,jk,jp_tem)
                  !! temperatur
                  ztmp(ji,jj) = tsb(ji,jj,jk,jp_tem)
               endif
               if (zsal(ji,jj) .lt. 0.0 .or. zsal(ji,jj) .gt. 45.0 ) then
                  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: S WARNING 2D, ', &
                     tsb(ji,jj,jk,jp_sal), tsn(ji,jj,jk,jp_sal), ' at (',    &
                     ji, ',', jj, ',', jk, ') at time', kt
               endif
# else
               !! implicit detrital carbon
               zdtc(ji,jj) = zdet(ji,jj) * xthetad
# endif
# if defined key_debug_medusa
               if (idf.eq.1) then
                  !! AXY (15/01/10)
                  if (trn(ji,jj,jk,jpdin).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR =',        &
                        trn(ji,jj,jk,jpdin)
                     IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpsil).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR =',        &
                        trn(ji,jj,jk,jpsil)
                     IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR @',        &
                        ji, jj, jk, kt
                  endif
#  if defined key_roam
                  if (trn(ji,jj,jk,jpdic).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR =',        &
                        trn(ji,jj,jk,jpdic)
                     IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpalk).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR =',        &
                        trn(ji,jj,jk,jpalk)
                     IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpoxy).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR =',        &
                        trn(ji,jj,jk,jpoxy)
                     IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR @',        &
                        ji, jj, jk, kt
                  endif
#  endif
               endif
# endif
# if defined key_debug_medusa
               !! report state variable values
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fthk(',jk,') = ', fse3t(ji,jj,jk)
                  IF (lwp) write (numout,*) 'zphn(',jk,') = ', zphn(ji,jj)
                  IF (lwp) write (numout,*) 'zphd(',jk,') = ', zphd(ji,jj)
                  IF (lwp) write (numout,*) 'zpds(',jk,') = ', zpds(ji,jj)
                  IF (lwp) write (numout,*) 'zzmi(',jk,') = ', zzmi(ji,jj)
                  IF (lwp) write (numout,*) 'zzme(',jk,') = ', zzme(ji,jj)
                  IF (lwp) write (numout,*) 'zdet(',jk,') = ', zdet(ji,jj)
                  IF (lwp) write (numout,*) 'zdin(',jk,') = ', zdin(ji,jj)
                  IF (lwp) write (numout,*) 'zsil(',jk,') = ', zsil(ji,jj)
                  IF (lwp) write (numout,*) 'zfer(',jk,') = ', zfer(ji,jj)
#  if defined key_roam
                  IF (lwp) write (numout,*) 'zdtc(',jk,') = ', zdtc(ji,jj)
                  IF (lwp) write (numout,*) 'zdic(',jk,') = ', zdic(ji,jj)
                  IF (lwp) write (numout,*) 'zalk(',jk,') = ', zalk(ji,jj)
                  IF (lwp) write (numout,*) 'zoxy(',jk,') = ', zoxy(ji,jj)                  
#  endif
               endif
# endif

# if defined key_debug_medusa
               if (idf.eq.1.AND.idfval.eq.1.AND.jk.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'dust      = ', dust(ji,jj)
               endif
# endif

               !! sum tracers for inventory checks
               IF( lk_iomput ) THEN
                  IF ( med_diag%INVTN%dgsave )   THEN
                     ftot_n(ji,jj)  = ftot_n(ji,jj) +                        &
                        (fse3t(ji,jj,jk) * ( zphn(ji,jj) + zphd(ji,jj) +     &
                                             zzmi(ji,jj) + zzme(ji,jj) +     &
                                             zdet(ji,jj) + zdin(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTSI%dgsave )  THEN
                     ftot_si(ji,jj) = ftot_si(ji,jj) +                       & 
                        (fse3t(ji,jj,jk) * ( zpds(ji,jj) + zsil(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTFE%dgsave )  THEN
                     ftot_fe(ji,jj) = ftot_fe(ji,jj) +                       & 
                        (fse3t(ji,jj,jk) * ( xrfn *                          &
                                            ( zphn(ji,jj) + zphd(ji,jj) +    &
                                              zzmi(ji,jj) + zzme(ji,jj) +    &
                                              zdet(ji,jj) ) + zfer(ji,jj) ) )
                  ENDIF
# if defined key_roam
                  IF ( med_diag%INVTC%dgsave )  THEN
                     ftot_c(ji,jj)  = ftot_c(ji,jj) + & 
                        (fse3t(ji,jj,jk) * ( (xthetapn * zphn(ji,jj)) +      &
                                             (xthetapd * zphd(ji,jj)) +      &
                                             (xthetazmi * zzmi(ji,jj)) +     &
                                             (xthetazme * zzme(ji,jj)) +     &
                                             zdtc(ji,jj) + zdic(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTALK%dgsave ) THEN
                     ftot_a(ji,jj)  = ftot_a(ji,jj) + (fse3t(ji,jj,jk) *     &
                                                       zalk(ji,jj))
                  ENDIF
                  IF ( med_diag%INVTO2%dgsave )  THEN
                     ftot_o2(ji,jj) = ftot_o2(ji,jj) + (fse3t(ji,jj,jk) *    &
                                                        zoxy(ji,jj))
                  ENDIF
                  !!
                  !! AXY (10/11/16): CMIP6 diagnostics
                  IF ( med_diag%INTDISSIC%dgsave ) THEN
                     intdissic(ji,jj) = intdissic(ji,jj) +                   &
                                        (fse3t(ji,jj,jk) * zdic(ji,jj))
                  ENDIF
                  IF ( med_diag%INTDISSIN%dgsave ) THEN
                     intdissin(ji,jj) = intdissin(ji,jj) +                   &
                                        (fse3t(ji,jj,jk) * zdin(ji,jj))
                  ENDIF
                  IF ( med_diag%INTDISSISI%dgsave ) THEN
                     intdissisi(ji,jj) = intdissisi(ji,jj) +                 &
                                         (fse3t(ji,jj,jk) * zsil(ji,jj))
                  ENDIF
                  IF ( med_diag%INTTALK%dgsave ) THEN
                     inttalk(ji,jj) = inttalk(ji,jj) +                       &
                                      (fse3t(ji,jj,jk) * zalk(ji,jj))
                  ENDIF
                  IF ( med_diag%O2min%dgsave ) THEN
                     if ( zoxy(ji,jj) < o2min(ji,jj) ) then
                        o2min(ji,jj)  = zoxy(ji,jj)
                        IF ( med_diag%ZO2min%dgsave ) THEN
                           !! layer midpoint
                           zo2min(ji,jj) = (fsdepw(ji,jj,jk) +               &
                                            fdep1(ji,jj)) / 2.0
                        ENDIF
                     endif
                  ENDIF
# endif
               ENDIF

               CALL flush(numout)

            ENDIF
         ENDDO
         ENDDO

         !!----------------------------------------------------------------
         !! Calculate air-sea gas exchange and river inputs
         !!----------------------------------------------------------------
         IF ( jk == 1 ) THEN
            call air_sea( kt )
         END IF

# if defined key_roam

! Moved from above - marc 21/4/17
! I think this should be moved into diagnostics at bottom - it
! doesn't like it is used anyway else - marc 21/4/17
         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !! AXY (11/11/16): CMIP6 oxygen saturation 3D diagnostic
               IF ( med_diag%O2SAT3%dgsave ) THEN
                  call oxy_sato( ztmp(ji,jj), zsal(ji,jj), f_o2sat3 )
                  o2sat3(ji, jj, jk) = f_o2sat3
               ENDIF
            ENDIF
         ENDDO
         ENDDO
# endif

         !!------------------------------------------------------------------
         !! Phytoplankton growth, zooplankton grazing and miscellaneous
         !! plankton losses. 
         !!------------------------------------------------------------------
         CALL plankton( jk )

         !!----------------------------------------------------------------
         !! Iron chemistry and scavenging
         !!----------------------------------------------------------------
         CALL iron_chem_scav( jk )

! Miscellaneous processes - marc

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!----------------------------------------------------------------------
               !! Miscellaneous
               !!----------------------------------------------------------------------
               !!
               !! diatom frustule dissolution
               fsdiss(ji,jj)  = xsdiss * zpds(ji,jj)

# if defined key_debug_medusa
               !! report miscellaneous calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fsdiss(',jk,')  = ', fsdiss(ji,jj)
               endif
# endif

               !!----------------------------------------------------------------------
               !! Slow detritus creation
               !!----------------------------------------------------------------------
               !! this variable integrates the creation of slow sinking detritus
               !! to allow the split between fast and slow detritus to be 
               !! diagnosed
               fslown(ji,jj)  = fdpn(ji,jj) + fdzmi(ji,jj) + ((1.0 - xfdfrac1) * fdpd(ji,jj)) + &
               ((1.0 - xfdfrac2) * fdzme(ji,jj)) + ((1.0 - xbetan) * (finmi(ji,jj) + finme(ji,jj)))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslownflux(ji,jj) = zdet(ji,jj) * vsed * 86400.
# if defined key_roam
               !!
               !! and the same for detrital carbon
               fslowc(ji,jj)  = (xthetapn * fdpn(ji,jj)) + (xthetazmi * fdzmi(ji,jj)) + &
               (xthetapd * (1.0 - xfdfrac1) * fdpd(ji,jj)) + &
               (xthetazme * (1.0 - xfdfrac2) * fdzme(ji,jj)) + &
               ((1.0 - xbetac) * (ficmi(ji,jj) + ficme(ji,jj)))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslowcflux(ji,jj) = zdtc(ji,jj) * vsed * 86400.
# endif

               !!----------------------------------------------------------------------
               !! Nutrient regeneration
               !! this variable integrates total nitrogen regeneration down the
               !! watercolumn; its value is stored and output as a 2D diagnostic;
               !! the corresponding dissolution flux of silicon (from sources
               !! other than fast detritus) is also integrated; note that,
               !! confusingly, the linear loss terms from plankton compartments
               !! are labelled as fdX2 when one might have expected fdX or fdX1
               !!----------------------------------------------------------------------
               !!
               !! nitrogen
               fregen(ji,jj)   = (( (xphi * (fgmipn(ji,jj) + fgmid(ji,jj))) +                &  ! messy feeding
               (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj))) +           &  ! messy feeding
               fmiexcr(ji,jj) + fmeexcr(ji,jj) + fdd(ji,jj) +                                &  ! excretion + D remin.
               fdpn2(ji,jj) + fdpd2(ji,jj) + fdzmi2(ji,jj) + fdzme2(ji,jj)) * fse3t(ji,jj,jk))                    ! linear mortality
               !!
               !! silicon
               fregensi(ji,jj) = (( fsdiss(ji,jj) + ((1.0 - xfdfrac1) * fdpds(ji,jj)) +      &  ! dissolution + non-lin. mortality
               ((1.0 - xfdfrac3) * fgmepds(ji,jj)) +                           &  ! egestion by zooplankton
               fdpds2(ji,jj)) * fse3t(ji,jj,jk))                                             ! linear mortality
# if defined key_roam
               !!
               !! carbon
! Doesn't look this is used - marc 10/4/17
!               fregenc(ji,jj)  = (( (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj))) +  &  ! messy feeding
!               (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) +     &  ! messy feeding
!               (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj))) +                       &  ! messy feeding
!               fmiresp(ji,jj) + fmeresp(ji,jj) + fddc(ji,jj) +                               &  ! respiration + D remin.
!               (xthetapn * fdpn2(ji,jj)) + (xthetapd * fdpd2(ji,jj)) +                &  ! linear mortality
!               (xthetazmi * fdzmi2(ji,jj)) + (xthetazme * fdzme2(ji,jj))) * fse3t(ji,jj,jk))        ! linear mortality
# endif

! MAYBE BUT A BREAK IN HERE, FAST-SINKINIG DETRITUS - marc 20/4/17 
! (miscellaneous processes is 79 lines)
            ENDIF
         ENDDO
         ENDDO

         !!------------------------------------------------------------------
         !! Detritus process
         !!------------------------------------------------------------------
         CALL detritus( jk, iball )

         !!------------------------------------------------------------------
         !! Updating tracers
         !!------------------------------------------------------------------
         CALL bio_medusa_update( kt, jk )

! Diagnostic update - marc

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

# if defined key_trc_diabio
               !!======================================================================
               !! LOCAL GRID CELL DIAGNOSTICS
               !!======================================================================
               !!
               !!----------------------------------------------------------------------
               !! Full diagnostics key_trc_diabio:
               !! LOBSTER and PISCES support full diagnistics option key_trc_diabio    
               !! which gives an option of FULL output of biological sourses and sinks.
               !! I cannot see any reason for doing this. If needed, it can be done
               !! as shown below.
               !!----------------------------------------------------------------------
               !!
               IF(lwp) WRITE(numout,*) ' MEDUSA does not support key_trc_diabio'
               !!               trbio(ji,jj,jk, 1) = fprn(ji,jj)
# endif

               IF( lk_iomput  .AND.  .NOT.  ln_diatrc  ) THEN
         !!----------------------------------------------------------------------
         !! Add in XML diagnostics stuff
         !!----------------------------------------------------------------------
         !!
         !! ** 2D diagnostics
#   if defined key_debug_medusa
                  IF (lwp) write (numout,*) 'trc_bio_medusa: diag in ij-jj-jk loop'
                  CALL flush(numout)
#   endif
                  IF ( med_diag%PRN%dgsave ) THEN
                      fprn2d(ji,jj) = fprn2d(ji,jj) + (fprn(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF ( med_diag%MPN%dgsave ) THEN
                      fdpn2d(ji,jj) = fdpn2d(ji,jj) + (fdpn(ji,jj)         * fse3t(ji,jj,jk))
                  ENDIF
                  IF ( med_diag%PRD%dgsave ) THEN
                      fprd2d(ji,jj) = fprd2d(ji,jj) + (fprd(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%MPD%dgsave ) THEN
                      fdpd2d(ji,jj) = fdpd2d(ji,jj) + (fdpd(ji,jj)         * fse3t(ji,jj,jk)) 
                  ENDIF
                  !  IF( med_diag%DSED%dgsave ) THEN
                  !      CALL iom_put( "DSED"  , ftot_n )
                  !  ENDIF
                  IF( med_diag%OPAL%dgsave ) THEN
                      fprds2d(ji,jj) = fprds2d(ji,jj) + (fprds(ji,jj) * zpds(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%OPALDISS%dgsave ) THEN
                      fsdiss2d(ji,jj) = fsdiss2d(ji,jj) + (fsdiss(ji,jj)  * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%GMIPn%dgsave ) THEN
                      fgmipn2d(ji,jj) = fgmipn2d(ji,jj) + (fgmipn(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMID%dgsave ) THEN
                      fgmid2d(ji,jj) = fgmid2d(ji,jj) + (fgmid(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%MZMI%dgsave ) THEN
                      fdzmi2d(ji,jj) = fdzmi2d(ji,jj) + (fdzmi(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMEPN%dgsave ) THEN
                      fgmepn2d(ji,jj) = fgmepn2d(ji,jj) + (fgmepn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMEPD%dgsave ) THEN
                      fgmepd2d(ji,jj) = fgmepd2d(ji,jj) + (fgmepd(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMEZMI%dgsave ) THEN
                      fgmezmi2d(ji,jj) = fgmezmi2d(ji,jj) + (fgmezmi(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMED%dgsave ) THEN
                      fgmed2d(ji,jj) = fgmed2d(ji,jj) + (fgmed(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%MZME%dgsave ) THEN
                      fdzme2d(ji,jj) = fdzme2d(ji,jj) + (fdzme(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  !  IF( med_diag%DEXP%dgsave ) THEN
                  !      CALL iom_put( "DEXP"  , ftot_n )
                  !  ENDIF
                  IF( med_diag%DETN%dgsave ) THEN
                      fslown2d(ji,jj) = fslown2d(ji,jj) + (fslown(ji,jj)  * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%MDET%dgsave ) THEN
                      fdd2d(ji,jj) = fdd2d(ji,jj) + (fdd(ji,jj)     * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%AEOLIAN%dgsave ) THEN
                      ffetop2d(ji,jj) = ffetop2d(ji,jj) + (ffetop(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%BENTHIC%dgsave ) THEN
                      ffebot2d(ji,jj) = ffebot2d(ji,jj) + (ffebot(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%SCAVENGE%dgsave ) THEN
                      ffescav2d(ji,jj) = ffescav2d(ji,jj) + (ffescav(ji,jj) * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%PN_JLIM%dgsave ) THEN
                      ! fjln2d(ji,jj) = fjln2d(ji,jj) + (fjln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                      fjln2d(ji,jj) = fjln2d(ji,jj) + (fjlim_pn(ji,jj) * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PN_NLIM%dgsave ) THEN
                      fnln2d(ji,jj) = fnln2d(ji,jj) + (fnln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PN_FELIM%dgsave ) THEN
                      ffln2d(ji,jj) = ffln2d(ji,jj) + (ffln2(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_JLIM%dgsave ) THEN
                      ! fjld2d(ji,jj) = fjld2d(ji,jj) + (fjld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                      fjld2d(ji,jj) = fjld2d(ji,jj) + (fjlim_pd(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_NLIM%dgsave ) THEN
                      fnld2d(ji,jj) = fnld2d(ji,jj) + (fnld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_FELIM%dgsave ) THEN
                      ffld2d(ji,jj) = ffld2d(ji,jj) + (ffld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_SILIM%dgsave ) THEN
                      fsld2d2(ji,jj) = fsld2d2(ji,jj) + (fsld2(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PDSILIM2%dgsave ) THEN
                      fsld2d(ji,jj) = fsld2d(ji,jj) + (fsld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  !! 
                  IF( med_diag%TOTREG_N%dgsave ) THEN
                      fregen2d(ji,jj) = fregen2d(ji,jj) + fregen(ji,jj)
                  ENDIF
                  IF( med_diag%TOTRG_SI%dgsave ) THEN
                      fregensi2d(ji,jj) = fregensi2d(ji,jj) + fregensi(ji,jj)
                  ENDIF
                  !! 
                  IF( med_diag%FASTN%dgsave ) THEN
                      ftempn2d(ji,jj) = ftempn2d(ji,jj) + (ftempn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTSI%dgsave ) THEN
                      ftempsi2d(ji,jj) = ftempsi2d(ji,jj) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTFE%dgsave ) THEN
                      ftempfe2d(ji,jj) =ftempfe2d(ji,jj)  + (ftempfe(ji,jj) * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%FASTC%dgsave ) THEN
                      ftempc2d(ji,jj) = ftempc2d(ji,jj) + (ftempc(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTCA%dgsave ) THEN
                      ftempca2d(ji,jj) = ftempca2d(ji,jj) + (ftempca(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  !! 
                  IF( med_diag%REMINN%dgsave ) THEN
                      freminn2d(ji,jj) = freminn2d(ji,jj) + (freminn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%REMINSI%dgsave ) THEN
                      freminsi2d(ji,jj) = freminsi2d(ji,jj) + (freminsi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%REMINFE%dgsave ) THEN
                      freminfe2d(ji,jj)= freminfe2d(ji,jj) + (freminfe(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%REMINC%dgsave ) THEN
                      freminc2d(ji,jj) = freminc2d(ji,jj) + (freminc(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%REMINCA%dgsave ) THEN
                      freminca2d(ji,jj) = freminca2d(ji,jj) + (freminca(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  !!
# if defined key_roam
                  !!
                  !! AXY (09/11/16): CMIP6 diagnostics
                  IF( med_diag%FD_NIT3%dgsave ) THEN
                     fd_nit3(ji,jj,jk) = ffastn(ji,jj)
                  ENDIF
                  IF( med_diag%FD_SIL3%dgsave ) THEN
                     fd_sil3(ji,jj,jk) = ffastsi(ji,jj)
                  ENDIF
                  IF( med_diag%FD_CAR3%dgsave ) THEN
                     fd_car3(ji,jj,jk) = ffastc(ji,jj)
                  ENDIF
                  IF( med_diag%FD_CAL3%dgsave ) THEN
                     fd_cal3(ji,jj,jk) = ffastca(ji,jj)
                  ENDIF
                  !!
                  IF (jk.eq.i0100) THEN
                     IF( med_diag%RR_0100%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF                     
                  ELSE IF (jk.eq.i0500) THEN 
                     IF( med_diag%RR_0500%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF                        
                  ELSE IF (jk.eq.i1000) THEN
                     IF( med_diag%RR_1000%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF
                  ELSE IF (jk.eq.mbathy(ji,jj)) THEN
                     IF( med_diag%IBEN_N%dgsave ) THEN
                        iben_n2d(ji,jj) = f_sbenin_n(ji,jj)  + f_fbenin_n(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_FE%dgsave ) THEN
                        iben_fe2d(ji,jj) = f_sbenin_fe(ji,jj) + f_fbenin_fe(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_C%dgsave ) THEN
                        iben_c2d(ji,jj) = f_sbenin_c(ji,jj)  + f_fbenin_c(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_SI%dgsave ) THEN
                        iben_si2d(ji,jj) = f_fbenin_si(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_CA%dgsave ) THEN
                        iben_ca2d(ji,jj) = f_fbenin_ca(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_N%dgsave ) THEN
                        oben_n2d(ji,jj) = f_benout_n(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_FE%dgsave ) THEN
                        oben_fe2d(ji,jj) = f_benout_fe(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_C%dgsave ) THEN
                        oben_c2d(ji,jj) = f_benout_c(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_SI%dgsave ) THEN
                        oben_si2d(ji,jj) = f_benout_si(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_CA%dgsave ) THEN
                        oben_ca2d(ji,jj) = f_benout_ca(ji,jj)
                     ENDIF
                     IF( med_diag%SFR_OCAL%dgsave ) THEN
                        sfr_ocal2d(ji,jj) = f3_omcal(ji,jj,jk)
                     ENDIF
                     IF( med_diag%SFR_OARG%dgsave ) THEN
                        sfr_oarg2d(ji,jj) =  f3_omarg(ji,jj,jk)
                     ENDIF
                     IF( med_diag%LYSO_CA%dgsave ) THEN
                        lyso_ca2d(ji,jj) = f_benout_lyso_ca(ji,jj)
                     ENDIF
                  ENDIF
                  !! end bathy-1 diags
                  !!
                  IF( med_diag%RIV_N%dgsave ) THEN
                     rivn2d(ji,jj) = rivn2d(ji,jj) +  (f_riv_loc_n(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_SI%dgsave ) THEN
                     rivsi2d(ji,jj) = rivsi2d(ji,jj) +  (f_riv_loc_si(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_C%dgsave ) THEN
                     rivc2d(ji,jj) = rivc2d(ji,jj) +  (f_riv_loc_c(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_ALK%dgsave ) THEN
                     rivalk2d(ji,jj) = rivalk2d(ji,jj) +  (f_riv_loc_alk(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%DETC%dgsave ) THEN
                     fslowc2d(ji,jj) = fslowc2d(ji,jj) + (fslowc(ji,jj)  * fse3t(ji,jj,jk))   
                  ENDIF
                  !! 
                  !!              
                  !!
                  IF( med_diag%PN_LLOSS%dgsave ) THEN
                     fdpn22d(ji,jj) = fdpn22d(ji,jj) + (fdpn2(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%PD_LLOSS%dgsave ) THEN
                     fdpd22d(ji,jj) = fdpd22d(ji,jj) + (fdpd2(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_LLOSS%dgsave ) THEN
                     fdzmi22d(ji,jj) = fdzmi22d(ji,jj) + (fdzmi2(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_LLOSS%dgsave ) THEN
                     fdzme22d(ji,jj) = fdzme22d(ji,jj) + (fdzme2(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_N%dgsave ) THEN
                     zimesn2d(ji,jj) = zimesn2d(ji,jj) +  &
                     (xphi * (fgmipn(ji,jj) + fgmid(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_D%dgsave ) THEN
                     zimesd2d(ji,jj) = zimesd2d(ji,jj) + & 
                     ((1. - xbetan) * finmi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_C%dgsave ) THEN
                     zimesc2d(ji,jj) = zimesc2d(ji,jj) + &
                     (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MESDC%dgsave ) THEN
                     zimesdc2d(ji,jj) = zimesdc2d(ji,jj) + &
                     ((1. - xbetac) * ficmi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_EXCR%dgsave ) THEN
                     ziexcr2d(ji,jj) = ziexcr2d(ji,jj) +  (fmiexcr(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_RESP%dgsave ) THEN
                     ziresp2d(ji,jj) = ziresp2d(ji,jj) +  (fmiresp(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_GROW%dgsave ) THEN
                     zigrow2d(ji,jj) = zigrow2d(ji,jj) + (fmigrow(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_N%dgsave ) THEN
                     zemesn2d(ji,jj) = zemesn2d(ji,jj) + &
                     (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_D%dgsave ) THEN
                     zemesd2d(ji,jj) = zemesd2d(ji,jj) + &
                     ((1. - xbetan) * finme(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_C%dgsave ) THEN
                     zemesc2d(ji,jj) = zemesc2d(ji,jj) +                         & 
                     (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) +  &
                     (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MESDC%dgsave ) THEN
                     zemesdc2d(ji,jj) = zemesdc2d(ji,jj) +  &
                     ((1. - xbetac) * ficme(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_EXCR%dgsave ) THEN
                     zeexcr2d(ji,jj) = zeexcr2d(ji,jj) + (fmeexcr(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_RESP%dgsave ) THEN
                     zeresp2d(ji,jj) = zeresp2d(ji,jj) + (fmeresp(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_GROW%dgsave ) THEN
                     zegrow2d(ji,jj) = zegrow2d(ji,jj) + (fmegrow(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%MDETC%dgsave ) THEN
                     mdetc2d(ji,jj) = mdetc2d(ji,jj) + (fddc(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMIDC%dgsave ) THEN
                     gmidc2d(ji,jj) = gmidc2d(ji,jj) + (fgmidc(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMEDC%dgsave ) THEN
                     gmedc2d(ji,jj) = gmedc2d(ji,jj) + (fgmedc(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  !!
# endif                   
                  !!
                  !! ** 3D diagnostics
                  IF( med_diag%TPP3%dgsave ) THEN
                     tpp3d(ji,jj,jk) =  (fprn(ji,jj) * zphn(ji,jj)) + (fprd(ji,jj) * zphd(ji,jj))
                     !CALL iom_put( "TPP3"  , tpp3d )
                  ENDIF
                  IF( med_diag%TPPD3%dgsave ) THEN
                     tppd3(ji,jj,jk) =  (fprd(ji,jj) * zphd(ji,jj))
                  ENDIF
                  
                  IF( med_diag%REMIN3N%dgsave ) THEN
                     remin3dn(ji,jj,jk) = fregen(ji,jj) + (freminn(ji,jj) * fse3t(ji,jj,jk)) !! remineralisation
                     !CALL iom_put( "REMIN3N"  , remin3dn )
                  ENDIF
                  !! IF( med_diag%PH3%dgsave ) THEN
                  !!     CALL iom_put( "PH3"  , f3_pH )
                  !! ENDIF
                  !! IF( med_diag%OM_CAL3%dgsave ) THEN
                  !!     CALL iom_put( "OM_CAL3"  , f3_omcal )
                  !! ENDIF
		  !! 
		  !! AXY (09/11/16): CMIP6 diagnostics
		  IF ( med_diag%DCALC3%dgsave   ) THEN
                     dcalc3(ji,jj,jk) = freminca(ji,jj)
                  ENDIF
		  IF ( med_diag%FEDISS3%dgsave  ) THEN
                     fediss3(ji,jj,jk) = ffetop(ji,jj)
                  ENDIF
		  IF ( med_diag%FESCAV3%dgsave  ) THEN
                     fescav3(ji,jj,jk) = ffescav(ji,jj)
                  ENDIF
		  IF ( med_diag%MIGRAZP3%dgsave ) THEN
                     migrazp3(ji,jj,jk) = fgmipn(ji,jj) * xthetapn
                  ENDIF
		  IF ( med_diag%MIGRAZD3%dgsave ) THEN
                     migrazd3(ji,jj,jk) = fgmidc(ji,jj)
                  ENDIF
		  IF ( med_diag%MEGRAZP3%dgsave ) THEN
                     megrazp3(ji,jj,jk) = (fgmepn(ji,jj) * xthetapn) + (fgmepd(ji,jj) * xthetapd)
                  ENDIF
		  IF ( med_diag%MEGRAZD3%dgsave ) THEN
                     megrazd3(ji,jj,jk) = fgmedc(ji,jj)
                  ENDIF
		  IF ( med_diag%MEGRAZZ3%dgsave ) THEN
                     megrazz3(ji,jj,jk) = (fgmezmi(ji,jj) * xthetazmi)
                  ENDIF
		  IF ( med_diag%PBSI3%dgsave    ) THEN
                     pbsi3(ji,jj,jk)    = (fprds(ji,jj) * zpds(ji,jj))
                  ENDIF
		  IF ( med_diag%PCAL3%dgsave    ) THEN
                     pcal3(ji,jj,jk)    = ftempca(ji,jj)
                  ENDIF
		  IF ( med_diag%REMOC3%dgsave   ) THEN
                     remoc3(ji,jj,jk)   = freminc(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMJ3%dgsave  ) THEN
                     ! pnlimj3(ji,jj,jk)  = fjln(ji,jj)
                     pnlimj3(ji,jj,jk)  = fjlim_pn(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMN3%dgsave  ) THEN
                     pnlimn3(ji,jj,jk)  = fnln(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMFE3%dgsave ) THEN
                     pnlimfe3(ji,jj,jk) = ffln2(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMJ3%dgsave  ) THEN
                     ! pdlimj3(ji,jj,jk)  = fjld(ji,jj)
                     pdlimj3(ji,jj,jk)  = fjlim_pd(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMN3%dgsave  ) THEN
                     pdlimn3(ji,jj,jk)  = fnld(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMFE3%dgsave ) THEN
                     pdlimfe3(ji,jj,jk) = ffld(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMSI3%dgsave ) THEN
                     pdlimsi3(ji,jj,jk) = fsld2(ji,jj)
                  ENDIF
                  !!
                  !! ** Without using iom_use
               ELSE IF( ln_diatrc ) THEN
#   if defined key_debug_medusa
                  IF (lwp) write (numout,*) 'trc_bio_medusa: diag in ij-jj-jk ln_diatrc'
                  CALL flush(numout)
#   endif
                  !!----------------------------------------------------------------------
                  !! Prepare 2D diagnostics
                  !!----------------------------------------------------------------------
                  !!
                  !! if ((kt / 240*240).eq.kt) then
                  !!    IF (lwp) write (*,*) '*******!MEDUSA DIAADD!*******',kt
                  !! endif     
                  trc2d(ji,jj,1)  =  ftot_n(ji,jj)                             !! nitrogen inventory
                  trc2d(ji,jj,2)  =  ftot_si(ji,jj)                            !! silicon  inventory
                  trc2d(ji,jj,3)  =  ftot_fe(ji,jj)                            !! iron     inventory
                  trc2d(ji,jj,4)  = trc2d(ji,jj,4)  + (fprn(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom production
                  trc2d(ji,jj,5)  = trc2d(ji,jj,5)  + (fdpn(ji,jj)         * fse3t(ji,jj,jk))    !! non-diatom non-grazing losses
                  trc2d(ji,jj,6)  = trc2d(ji,jj,6)  + (fprd(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom production
                  trc2d(ji,jj,7)  = trc2d(ji,jj,7)  + (fdpd(ji,jj)         * fse3t(ji,jj,jk))    !! diatom non-grazing losses
                  !! diagnostic field  8 is (ostensibly) supplied by trcsed.F            
                  trc2d(ji,jj,9)  = trc2d(ji,jj,9)  + (fprds(ji,jj) * zpds(ji,jj) * fse3t(ji,jj,jk))    !! diatom silicon production
                  trc2d(ji,jj,10) = trc2d(ji,jj,10) + (fsdiss(ji,jj)  * fse3t(ji,jj,jk))         !! diatom silicon dissolution
                  trc2d(ji,jj,11) = trc2d(ji,jj,11) + (fgmipn(ji,jj)  * fse3t(ji,jj,jk))         !! microzoo grazing on non-diatoms
                  trc2d(ji,jj,12) = trc2d(ji,jj,12) + (fgmid(ji,jj)   * fse3t(ji,jj,jk))         !! microzoo grazing on detrital nitrogen
                  trc2d(ji,jj,13) = trc2d(ji,jj,13) + (fdzmi(ji,jj)   * fse3t(ji,jj,jk))         !! microzoo non-grazing losses
                  trc2d(ji,jj,14) = trc2d(ji,jj,14) + (fgmepn(ji,jj)  * fse3t(ji,jj,jk))         !! mesozoo  grazing on non-diatoms
                  trc2d(ji,jj,15) = trc2d(ji,jj,15) + (fgmepd(ji,jj)  * fse3t(ji,jj,jk))         !! mesozoo  grazing on diatoms
                  trc2d(ji,jj,16) = trc2d(ji,jj,16) + (fgmezmi(ji,jj) * fse3t(ji,jj,jk))         !! mesozoo  grazing on microzoo
                  trc2d(ji,jj,17) = trc2d(ji,jj,17) + (fgmed(ji,jj)   * fse3t(ji,jj,jk))         !! mesozoo  grazing on detrital nitrogen
                  trc2d(ji,jj,18) = trc2d(ji,jj,18) + (fdzme(ji,jj)   * fse3t(ji,jj,jk))         !! mesozoo  non-grazing losses
                  !! diagnostic field 19 is (ostensibly) supplied by trcexp.F
                  trc2d(ji,jj,20) = trc2d(ji,jj,20) + (fslown(ji,jj)  * fse3t(ji,jj,jk))         !! slow sinking detritus N production
                  trc2d(ji,jj,21) = trc2d(ji,jj,21) + (fdd(ji,jj)     * fse3t(ji,jj,jk))         !! detrital remineralisation
                  trc2d(ji,jj,22) = trc2d(ji,jj,22) + (ffetop(ji,jj)  * fse3t(ji,jj,jk))         !! aeolian  iron addition
                  trc2d(ji,jj,23) = trc2d(ji,jj,23) + (ffebot(ji,jj)  * fse3t(ji,jj,jk))         !! seafloor iron addition
                  trc2d(ji,jj,24) = trc2d(ji,jj,24) + (ffescav(ji,jj) * fse3t(ji,jj,jk))         !! "free" iron scavenging
                  trc2d(ji,jj,25) = trc2d(ji,jj,25) + (fjlim_pn(ji,jj) * zphn(ji,jj) * fse3t(ji,jj,jk)) !! non-diatom J  limitation term 
                  trc2d(ji,jj,26) = trc2d(ji,jj,26) + (fnln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom N  limitation term 
                  trc2d(ji,jj,27) = trc2d(ji,jj,27) + (ffln2(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom Fe limitation term 
                  trc2d(ji,jj,28) = trc2d(ji,jj,28) + (fjlim_pd(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) !! diatom     J  limitation term 
                  trc2d(ji,jj,29) = trc2d(ji,jj,29) + (fnld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     N  limitation term 
                  trc2d(ji,jj,30) = trc2d(ji,jj,30) + (ffld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Fe limitation term 
                  trc2d(ji,jj,31) = trc2d(ji,jj,31) + (fsld2(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Si limitation term 
                  trc2d(ji,jj,32) = trc2d(ji,jj,32) + (fsld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Si uptake limitation term
                  if (jk.eq.i0100) trc2d(ji,jj,33) = fslownflux(ji,jj)         !! slow detritus flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,34) = fslownflux(ji,jj)         !! slow detritus flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,35) = fslownflux(ji,jj)         !! slow detritus flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,36) = fslownflux(ji,jj)         !! slow detritus flux at 1000 m
                  trc2d(ji,jj,37) = trc2d(ji,jj,37) + fregen(ji,jj)                   !! non-fast N  full column regeneration
                  trc2d(ji,jj,38) = trc2d(ji,jj,38) + fregensi(ji,jj)                 !! non-fast Si full column regeneration
                  if (jk.eq.i0100) trc2d(ji,jj,39) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,40) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,41) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,42) = trc2d(ji,jj,37)           !! non-fast N  regeneration to 1000 m
                  trc2d(ji,jj,43) = trc2d(ji,jj,43) + (ftempn(ji,jj)  * fse3t(ji,jj,jk))         !! fast sinking detritus N production
                  trc2d(ji,jj,44) = trc2d(ji,jj,44) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus Si production
                  trc2d(ji,jj,45) = trc2d(ji,jj,45) + (ftempfe(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus Fe production
                  trc2d(ji,jj,46) = trc2d(ji,jj,46) + (ftempc(ji,jj)  * fse3t(ji,jj,jk))         !! fast sinking detritus C production
                  trc2d(ji,jj,47) = trc2d(ji,jj,47) + (ftempca(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus CaCO3 production
                  if (jk.eq.i0100) trc2d(ji,jj,48) = ffastn(ji,jj)             !! fast detritus N  flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,49) = ffastn(ji,jj)             !! fast detritus N  flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,50) = ffastn(ji,jj)             !! fast detritus N  flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,51) = ffastn(ji,jj)             !! fast detritus N  flux at 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,52) = fregenfast(ji,jj)         !! N  regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,53) = fregenfast(ji,jj)         !! N  regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,54) = fregenfast(ji,jj)         !! N  regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,55) = fregenfast(ji,jj)         !! N  regeneration to 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,56) = ffastsi(ji,jj)            !! fast detritus Si flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,57) = ffastsi(ji,jj)            !! fast detritus Si flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,58) = ffastsi(ji,jj)            !! fast detritus Si flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,59) = ffastsi(ji,jj)            !! fast detritus Si flux at 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,60) = fregenfastsi(ji,jj)       !! Si regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,61) = fregenfastsi(ji,jj)       !! Si regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,62) = fregenfastsi(ji,jj)       !! Si regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,63) = fregenfastsi(ji,jj)       !! Si regeneration to 1000 m
                  trc2d(ji,jj,64) = trc2d(ji,jj,64) + (freminn(ji,jj)  * fse3t(ji,jj,jk))        !! sum of fast-sinking N  fluxes
                  trc2d(ji,jj,65) = trc2d(ji,jj,65) + (freminsi(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Si fluxes
                  trc2d(ji,jj,66) = trc2d(ji,jj,66) + (freminfe(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Fe fluxes
                  trc2d(ji,jj,67) = trc2d(ji,jj,67) + (freminc(ji,jj)  * fse3t(ji,jj,jk))        !! sum of fast-sinking C  fluxes
                  trc2d(ji,jj,68) = trc2d(ji,jj,68) + (freminca(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Ca fluxes
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,69) = fsedn(ji,jj)                                   !! N  sedimentation flux                                  
                     trc2d(ji,jj,70) = fsedsi(ji,jj)                                  !! Si sedimentation flux
                     trc2d(ji,jj,71) = fsedfe(ji,jj)                                  !! Fe sedimentation flux
                     trc2d(ji,jj,72) = fsedc(ji,jj)                                   !! C  sedimentation flux
                     trc2d(ji,jj,73) = fsedca(ji,jj)                                  !! Ca sedimentation flux
                  endif
                  if (jk.eq.1)  trc2d(ji,jj,74) = qsr(ji,jj)
                  if (jk.eq.1)  trc2d(ji,jj,75) = xpar(ji,jj,jk)
                  !! if (jk.eq.1)  trc2d(ji,jj,75) = real(iters(ji,jj))
                  !! diagnostic fields 76 to 80 calculated below
                  trc2d(ji,jj,81) = trc2d(ji,jj,81) + fprn_ml(ji,jj)           !! mixed layer non-diatom production
                  trc2d(ji,jj,82) = trc2d(ji,jj,82) + fprd_ml(ji,jj)           !! mixed layer     diatom production
# if defined key_gulf_finland
                  if (jk.eq.1)  trc2d(ji,jj,83) = real(ibio_switch)            !! Gulf of Finland check
# else
                  trc2d(ji,jj,83) = ocal_ccd(ji,jj)                            !! calcite CCD depth
# endif
                  trc2d(ji,jj,84) = fccd(ji,jj)                                !! last model level above calcite CCD depth
                  if (jk.eq.1)     trc2d(ji,jj,85) = xFree(ji,jj)              !! surface "free" iron
                  if (jk.eq.i0200) trc2d(ji,jj,86) = xFree(ji,jj)              !! "free" iron at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,87) = xFree(ji,jj)              !! "free" iron at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,88) = xFree(ji,jj)              !! "free" iron at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,89) = xFree(ji,jj)              !! "free" iron at 1000 m
                  !! AXY (27/06/12): extract "euphotic depth"
                  if (jk.eq.1)     trc2d(ji,jj,90) = xze(ji,jj)
                  !! 
# if defined key_roam
                  !! ROAM provisionally has access to a further 20 2D diagnostics
                  if (jk .eq. 1) then
                     trc2d(ji,jj,91)  = trc2d(ji,jj,91)  + wndm(ji,jj)              !! surface wind
                     trc2d(ji,jj,92)  = trc2d(ji,jj,92)  + f_pco2atm(ji,jj)           !! atmospheric pCO2
                     trc2d(ji,jj,93)  = trc2d(ji,jj,93)  + f_ph(ji,jj)                !! ocean pH
                     trc2d(ji,jj,94)  = trc2d(ji,jj,94)  + f_pco2w(ji,jj)             !! ocean pCO2
                     trc2d(ji,jj,95)  = trc2d(ji,jj,95)  + f_h2co3(ji,jj)             !! ocean H2CO3 conc.
                     trc2d(ji,jj,96)  = trc2d(ji,jj,96)  + f_hco3(ji,jj)              !! ocean HCO3 conc.
                     trc2d(ji,jj,97)  = trc2d(ji,jj,97)  + f_co3(ji,jj)               !! ocean CO3 conc.
                     trc2d(ji,jj,98)  = trc2d(ji,jj,98)  + f_co2flux(ji,jj)           !! air-sea CO2 flux
                     trc2d(ji,jj,99)  = trc2d(ji,jj,99)  + f_omcal(ji,jj)      !! ocean omega calcite 
                     trc2d(ji,jj,100) = trc2d(ji,jj,100) + f_omarg(ji,jj)      !! ocean omega aragonite
                     trc2d(ji,jj,101) = trc2d(ji,jj,101) + f_TDIC(ji,jj)              !! ocean TDIC
                     trc2d(ji,jj,102) = trc2d(ji,jj,102) + f_TALK(ji,jj)              !! ocean TALK
                     trc2d(ji,jj,103) = trc2d(ji,jj,103) + f_kw660(ji,jj)             !! surface kw660
                     trc2d(ji,jj,104) = trc2d(ji,jj,104) + f_pp0(ji,jj)               !! surface pressure
                     trc2d(ji,jj,105) = trc2d(ji,jj,105) + f_o2flux(ji,jj)            !! air-sea O2 flux
                     trc2d(ji,jj,106) = trc2d(ji,jj,106) + f_o2sat(ji,jj)             !! ocean O2 saturation
                     trc2d(ji,jj,107) = f2_ccd_cal(ji,jj)                      !! depth calcite CCD
                     trc2d(ji,jj,108) = f2_ccd_arg(ji,jj)                      !! depth aragonite CCD
                  endif
                  if (jk .eq. mbathy(ji,jj)) then
                     trc2d(ji,jj,109) = f3_omcal(ji,jj,jk)                     !! seafloor omega calcite
                     trc2d(ji,jj,110) = f3_omarg(ji,jj,jk)                     !! seafloor omega aragonite
                  endif
                  !! diagnostic fields 111 to 117 calculated below
                  if (jk.eq.i0100) trc2d(ji,jj,118) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at  100 m
                  if (jk.eq.i0500) trc2d(ji,jj,119) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,120) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at 1000 m
                  !! AXY (18/01/12): benthic flux diagnostics
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,121) = f_sbenin_n(ji,jj)  + f_fbenin_n(ji,jj)
                     trc2d(ji,jj,122) = f_sbenin_fe(ji,jj) + f_fbenin_fe(ji,jj)
                     trc2d(ji,jj,123) = f_sbenin_c(ji,jj)  + f_fbenin_c(ji,jj)
                     trc2d(ji,jj,124) = f_fbenin_si(ji,jj)
                     trc2d(ji,jj,125) = f_fbenin_ca(ji,jj)
                     trc2d(ji,jj,126) = f_benout_n(ji,jj)
                     trc2d(ji,jj,127) = f_benout_fe(ji,jj)
                     trc2d(ji,jj,128) = f_benout_c(ji,jj)
                     trc2d(ji,jj,129) = f_benout_si(ji,jj)
                     trc2d(ji,jj,130) = f_benout_ca(ji,jj)
                  endif
                  !! diagnostics fields 131 to 135 calculated below
                  trc2d(ji,jj,136) = f_runoff(ji,jj)
                  !! AXY (19/07/12): amended to allow for riverine nutrient addition below surface
                  trc2d(ji,jj,137) = trc2d(ji,jj,137) + (f_riv_loc_n(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,138) = trc2d(ji,jj,138) + (f_riv_loc_si(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,139) = trc2d(ji,jj,139) + (f_riv_loc_c(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,140) = trc2d(ji,jj,140) + (f_riv_loc_alk(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,141) = trc2d(ji,jj,141) + (fslowc(ji,jj)  * fse3t(ji,jj,jk))       !! slow sinking detritus C production
                  if (jk.eq.i0100) trc2d(ji,jj,142) = fslowcflux(ji,jj)        !! slow detritus flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,143) = fslowcflux(ji,jj)        !! slow detritus flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,144) = fslowcflux(ji,jj)        !! slow detritus flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,145) = fslowcflux(ji,jj)        !! slow detritus flux at 1000 m
                  trc2d(ji,jj,146)  = trc2d(ji,jj,146)  + ftot_c(ji,jj)        !! carbon     inventory
                  trc2d(ji,jj,147)  = trc2d(ji,jj,147)  + ftot_a(ji,jj)        !! alkalinity inventory
                  trc2d(ji,jj,148)  = trc2d(ji,jj,148)  + ftot_o2(ji,jj)       !! oxygen     inventory
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,149) = f_benout_lyso_ca(ji,jj)
                  endif
                  trc2d(ji,jj,150) = fcomm_resp(ji,jj) * fse3t(ji,jj,jk)                  !! community respiration
		  !!
		  !! AXY (14/02/14): a Valentines Day gift to BASIN - a shedload of new
                  !!                 diagnostics that they'll most likely never need!
                  !!                 (actually, as with all such gifts, I'm giving them
                  !!                 some things I'd like myself!)
                  !! 
                  !! ----------------------------------------------------------------------
                  !! linear losses
                  !! non-diatom
                  trc2d(ji,jj,151) = trc2d(ji,jj,151) + (fdpn2(ji,jj)  * fse3t(ji,jj,jk))
                  !! diatom
                  trc2d(ji,jj,152) = trc2d(ji,jj,152) + (fdpd2(ji,jj)  * fse3t(ji,jj,jk))
                  !! microzooplankton
                  trc2d(ji,jj,153) = trc2d(ji,jj,153) + (fdzmi2(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton
                  trc2d(ji,jj,154) = trc2d(ji,jj,154) + (fdzme2(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! microzooplankton grazing
                  !! microzooplankton messy -> N
                  trc2d(ji,jj,155) = trc2d(ji,jj,155) + (xphi * (fgmipn(ji,jj) + fgmid(ji,jj)) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> D
                  trc2d(ji,jj,156) = trc2d(ji,jj,156) + ((1. - xbetan) * finmi(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> DIC
                  trc2d(ji,jj,157) = trc2d(ji,jj,157) + (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj)) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> Dc
                  trc2d(ji,jj,158) = trc2d(ji,jj,158) + ((1. - xbetac) * ficmi(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton excretion
                  trc2d(ji,jj,159) = trc2d(ji,jj,159) + (fmiexcr(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton respiration
                  trc2d(ji,jj,160) = trc2d(ji,jj,160) + (fmiresp(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton growth
                  trc2d(ji,jj,161) = trc2d(ji,jj,161) + (fmigrow(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! mesozooplankton grazing
                  !! mesozooplankton messy -> N
                  trc2d(ji,jj,162) = trc2d(ji,jj,162) + (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj)) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> D
                  trc2d(ji,jj,163) = trc2d(ji,jj,163) + ((1. - xbetan) * finme(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> DIC
                  trc2d(ji,jj,164) = trc2d(ji,jj,164) + (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) + &
                  &                  (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj)) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> Dc
                  trc2d(ji,jj,165) = trc2d(ji,jj,165) + ((1. - xbetac) * ficme(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton excretion
                  trc2d(ji,jj,166) = trc2d(ji,jj,166) + (fmeexcr(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton respiration
                  trc2d(ji,jj,167) = trc2d(ji,jj,167) + (fmeresp(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton growth
                  trc2d(ji,jj,168) = trc2d(ji,jj,168) + (fmegrow(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! miscellaneous
                  trc2d(ji,jj,169) = trc2d(ji,jj,169) + (fddc(ji,jj)    * fse3t(ji,jj,jk)) !! detrital C remineralisation
                  trc2d(ji,jj,170) = trc2d(ji,jj,170) + (fgmidc(ji,jj)  * fse3t(ji,jj,jk)) !! microzoo grazing on detrital carbon
                  trc2d(ji,jj,171) = trc2d(ji,jj,171) + (fgmedc(ji,jj)  * fse3t(ji,jj,jk)) !! mesozoo  grazing on detrital carbon
                  !!
                  !! ----------------------------------------------------------------------
		  !!
		  !! AXY (23/10/14): extract primary production related surface fields to
		  !!                 deal with diel cycle issues; hijacking BASIN 150m
		  !!                 diagnostics to do so (see commented out diagnostics
		  !!                 below this section)
		  !!
                  !! extract relevant BASIN fields at 150m
                  if (jk .eq. i0150) then
                     trc2d(ji,jj,172) = trc2d(ji,jj,4)    !! Pn PP
                     trc2d(ji,jj,173) = trc2d(ji,jj,151)  !! Pn linear loss
                     trc2d(ji,jj,174) = trc2d(ji,jj,5)    !! Pn non-linear loss
                     trc2d(ji,jj,175) = trc2d(ji,jj,11)   !! Pn grazing to Zmi
                     trc2d(ji,jj,176) = trc2d(ji,jj,14)   !! Pn grazing to Zme
                     trc2d(ji,jj,177) = trc2d(ji,jj,6)    !! Pd PP
                     trc2d(ji,jj,178) = trc2d(ji,jj,152)  !! Pd linear loss
                     trc2d(ji,jj,179) = trc2d(ji,jj,7)    !! Pd non-linear loss
                     trc2d(ji,jj,180) = trc2d(ji,jj,15)   !! Pd grazing to Zme
                     trc2d(ji,jj,181) = trc2d(ji,jj,12)   !! Zmi grazing on D
                     trc2d(ji,jj,182) = trc2d(ji,jj,170)  !! Zmi grazing on Dc
                     trc2d(ji,jj,183) = trc2d(ji,jj,155)  !! Zmi messy feeding loss to N
                     trc2d(ji,jj,184) = trc2d(ji,jj,156)  !! Zmi messy feeding loss to D
                     trc2d(ji,jj,185) = trc2d(ji,jj,157)  !! Zmi messy feeding loss to DIC
                     trc2d(ji,jj,186) = trc2d(ji,jj,158)  !! Zmi messy feeding loss to Dc
                     trc2d(ji,jj,187) = trc2d(ji,jj,159)  !! Zmi excretion
                     trc2d(ji,jj,188) = trc2d(ji,jj,160)  !! Zmi respiration
                     trc2d(ji,jj,189) = trc2d(ji,jj,161)  !! Zmi growth
                     trc2d(ji,jj,190) = trc2d(ji,jj,153)  !! Zmi linear loss
                     trc2d(ji,jj,191) = trc2d(ji,jj,13)   !! Zmi non-linear loss
                     trc2d(ji,jj,192) = trc2d(ji,jj,16)   !! Zmi grazing to Zme
                     trc2d(ji,jj,193) = trc2d(ji,jj,17)   !! Zme grazing on D
                     trc2d(ji,jj,194) = trc2d(ji,jj,171)  !! Zme grazing on Dc
                     trc2d(ji,jj,195) = trc2d(ji,jj,162)  !! Zme messy feeding loss to N
                     trc2d(ji,jj,196) = trc2d(ji,jj,163)  !! Zme messy feeding loss to D
                     trc2d(ji,jj,197) = trc2d(ji,jj,164)  !! Zme messy feeding loss to DIC
                     trc2d(ji,jj,198) = trc2d(ji,jj,165)  !! Zme messy feeding loss to Dc
                     trc2d(ji,jj,199) = trc2d(ji,jj,166)  !! Zme excretion
                     trc2d(ji,jj,200) = trc2d(ji,jj,167)  !! Zme respiration
                     trc2d(ji,jj,201) = trc2d(ji,jj,168)  !! Zme growth
                     trc2d(ji,jj,202) = trc2d(ji,jj,154)  !! Zme linear loss
                     trc2d(ji,jj,203) = trc2d(ji,jj,18)   !! Zme non-linear loss
                     trc2d(ji,jj,204) = trc2d(ji,jj,20)   !! Slow detritus production, N
                     trc2d(ji,jj,205) = trc2d(ji,jj,21)   !! Slow detritus remineralisation, N
                     trc2d(ji,jj,206) = trc2d(ji,jj,141)  !! Slow detritus production, C
                     trc2d(ji,jj,207) = trc2d(ji,jj,169)  !! Slow detritus remineralisation, C
                     trc2d(ji,jj,208) = trc2d(ji,jj,43)   !! Fast detritus production, N
                     trc2d(ji,jj,209) = trc2d(ji,jj,21)   !! Fast detritus remineralisation, N
                     trc2d(ji,jj,210) = trc2d(ji,jj,64)   !! Fast detritus production, C
                     trc2d(ji,jj,211) = trc2d(ji,jj,67)   !! Fast detritus remineralisation, C
                     trc2d(ji,jj,212) = trc2d(ji,jj,150)  !! Community respiration
                     trc2d(ji,jj,213) = fslownflux(ji,jj) !! Slow detritus N flux at 150 m
                     trc2d(ji,jj,214) = fslowcflux(ji,jj) !! Slow detritus C flux at 150 m
                     trc2d(ji,jj,215) = ffastn(ji,jj)     !! Fast detritus N flux at 150 m
                     trc2d(ji,jj,216) = ffastc(ji,jj)     !! Fast detritus C flux at 150 m
                  endif
                  !! 
                  !! Jpalm (11-08-2014)
                  !! Add UKESM1 diagnoatics 
                  !!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
                  if ((jk .eq. 1) .and.( jdms.eq.1)) then
                     trc2d(ji,jj,221) = dms_surf(ji,jj)          !! DMS surface concentration 
                     !! AXY (13/03/15): add in other DMS estimates
                     trc2d(ji,jj,222) = dms_andr(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,223) = dms_simo(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,224) = dms_aran(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,225) = dms_hall(ji,jj)          !! DMS surface concentration 
                  endif
# endif
                  !! other possible future diagnostics include:
                  !!   - integrated tracer values (esp. biological)
                  !!   - mixed layer tracer values
                  !!   - sub-surface chlorophyll maxima (plus depth)
                  !!   - different mixed layer depth criteria (T, sigma, var. sigma)

                  !!----------------------------------------------------------------------
                  !! Prepare 3D diagnostics
                  !!----------------------------------------------------------------------
                  !!
                  trc3d(ji,jj,jk,1)  = ((fprn(ji,jj) + fprd(ji,jj)) * zphn(ji,jj))     !! primary production  
                  trc3d(ji,jj,jk,2)  = fslownflux(ji,jj) + ffastn(ji,jj) !! detrital flux
                  trc3d(ji,jj,jk,3)  = fregen(ji,jj) + (freminn(ji,jj) * fse3t(ji,jj,jk))  !! remineralisation
# if defined key_roam
                  trc3d(ji,jj,jk,4)  = f3_pH(ji,jj,jk)            !! pH
                  trc3d(ji,jj,jk,5)  = f3_omcal(ji,jj,jk)         !! omega calcite
# else
                  trc3d(ji,jj,jk,4)  = ffastsi(ji,jj)             !! fast Si flux
# endif
             ENDIF   ! end of ln_diatrc option
             !! CLOSE wet point IF..THEN loop
            endif
         !! CLOSE horizontal loops
         ENDDO
         ENDDO
         !!
             IF( lk_iomput  .AND.  .NOT.  ln_diatrc  ) THEN

                !!-------------------------------------------------------
                !! 2d specific k level diags
                !!-------------------------------------------------------
                CALL bio_medusa_diag_slice( jk )

              ENDIF
      !! CLOSE vertical loop
      ENDDO

      !!----------------------------------------------------------------------
      !! Final calculations for diagnostics
      !!----------------------------------------------------------------------
      CALL bio_medusa_fin( kt )

# if defined key_trc_diabio
       !! Lateral boundary conditions on trcbio
       DO jn=1,jp_medusa_trd
          CALL lbc_lnk(trbio(:,:,1,jn),'T',1. )
       ENDDO 
# endif

# if defined key_debug_medusa
       IF(lwp) WRITE(numout,*) ' MEDUSA exiting trc_bio_medusa at kt =', kt
       CALL flush(numout)
# endif

   END SUBROUTINE trc_bio_medusa

#else
   !!======================================================================
   !!  Dummy module :                                   No MEDUSA bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE trc_bio_medusa( kt )                   ! Empty routine
      INTEGER, INTENT( in ) ::   kt
      WRITE(*,*) 'trc_bio_medusa: You should not have seen this print! error?', kt
   END SUBROUTINE trc_bio_medusa
#endif 

   !!======================================================================
END MODULE  trcbio_medusa
